!#========================================================================
!#                                                                       #
!# Software Name : PHASE/0 2019.01                                       #
!#                                                                       #
!#      Sub Routine : Prep_for_Calc_Epsilon                              #
!#                                                                       #
!#                                Written by T. Hamada 2006/4/28         #
!#                                                                       #
!#      Contact address :  IIS,The University of Tokyo RSS21 project     #
!#                                                                       #
!#"Multiscale Simulation System for Functional Analysis of Nanomaterials"#
!#                                                                       #
!#========================================================================
!
! $Id: Preparation_for_Calc_Epsilon.f90 376 2014-06-17 07:48:31Z jkoga $
   subroutine Prep_for_Calc_Epsilon
     use m_Epsilon_ek,            only : gen_k_points_eps_ek &
        &                              , calc_tm_square_eps_ek
     call gen_k_points_eps_ek
     call calc_tm_square_eps_ek
   end subroutine Prep_for_Calc_Epsilon
