!#========================================================================
!#                                                                       #
!# Software Name : PHASE/0 2019.01                                       #
!#                                                                       #
!#      Sub Routine : Calc_Epsilon                                       #
!#                                                                       #
!#                                Written by T. Hamada 2006/4/28         #
!#                                                                       #
!#      Contact address :  IIS,The University of Tokyo RSS21 project     #
!#                                                                       #
!#"Multiscale Simulation System for Functional Analysis of Nanomaterials"#
!#                                                                       #
!#========================================================================
!
! $Id: Calc_Epsilon.f90 409 2014-10-27 09:24:52Z jkoga $
   subroutine Calc_Epsilon
     use m_Epsilon_ek,               only : BZintegration_eps, calc_drude_eps, kkt_eps, optics_eps

! ========= KT_add ======== 13.0S
     use m_Const_Parameters, only : ON
     use m_Epsilon_ek,      only :  corelevel_eps
     use m_Control_Parameters,  only : sw_corelevel_spectrum
! ========================= 13.0S

     call BZintegration_eps
     call kkt_eps

! ======== KT_mod ========= 13.0S
!     call calc_drude_eps
!     call optics_eps
!
     if ( sw_corelevel_spectrum == ON ) then
        call corelevel_eps
     else
        call calc_drude_eps
        call optics_eps
! ========================= 13.0S
     endif

   end subroutine Calc_Epsilon
